package net.eduvax.heml;

import java.io.IOException;
import java.io.OutputStream;
import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;
import net.eduvax.xml.SaxWriter;
import org.xml.sax.SAXException;

public class XmlWriter implements ParserCallback {
	private String _paraElem="p";
	private String _enumElem="li";
	private String _indentElem="ul";
	private String _cDataElem="pre";
	private SaxWriter _writer;
    private Vector<Exception> _exceptions;
	private String _elemName;
	private String _rowName;
	private int _rowStyle;
	private Iterable<String> _fieldsName;
	public XmlWriter(OutputStream out) {
		_writer=new SaxWriter(out);
        _exceptions=new Vector<Exception>();
	}
    public Collection<Exception> getErrors() {
        return _exceptions;
    }
    public boolean hasError() {
        return _exceptions.size()>0;
    }
	public void openElement(String name) {
		_elemName=name;
	}
	public void closeElement() {
        try {
		    _writer.endElement();
        }
        catch (SAXException ex) {
            _exceptions.add(ex);
        }
	}
	public void addAttribute(String name,String value) {
		_writer.addAttribute(name,value);
	}
	public void endAttributes() {
        try {
		    _writer.startElement(_elemName);
        }
        catch (SAXException ex) {
            _exceptions.add(ex);
        }
	}
	public void addText(String text) {
        try {
		    _writer.addText(text);
        }
        catch (SAXException ex) {
            _exceptions.add(ex);
        }
	}
	public void openPara() {
        try {
		    _writer.startElement(_paraElem);
        }
        catch (SAXException ex) {
            _exceptions.add(ex);
        }
	}
	public void closePara() {
        try {
		    _writer.endElement();
        }
        catch (SAXException ex) {
            _exceptions.add(ex);
        }
	}
	public void openEnum() {
        try {
		    _writer.startElement(_enumElem);
        }
        catch (SAXException ex) {
            _exceptions.add(ex);
        }
	}
	public void closeEnum() {
        try {
		    _writer.endElement();
        }
        catch (SAXException ex) {
            _exceptions.add(ex);
        }
	}
	public void openIndent() {
        try {
		    _writer.startElement(_indentElem);
        }
        catch (SAXException ex) {
            _exceptions.add(ex);
        }
	}
	public void closeIndent() {
        try {
		    _writer.endElement();
        }
        catch (SAXException ex) {
            _exceptions.add(ex);
        }
	}
	public void addComment(String comment) {
	}
	public void addCData(String cData) {
        try {
			_writer.startElement(_cDataElem);
			_writer.addText(cData);
		    _writer.endElement();
        }
        catch (SAXException ex) {
            _exceptions.add(ex);
        }
	}
    public void closeDocument() {
    }
    public void stateChanged(Parser.State s) {
    }

	public void openTable(int rowStyle,String rowName,Iterable<String> fieldsName) {
		_rowStyle=rowStyle;
		_rowName=rowName;
		_fieldsName=fieldsName;
	}
	public void closeTable() {
	}
	public void addRow(Iterable<String> fieldsValue) {
		try {
			if (_rowStyle==ParserCallback.ROW_ELEM) {
				_writer.startElement(_rowName); 
				Iterator<String> it=_fieldsName.iterator();
				int i=0;
                int qfn=0;
				String fName=null;
				for (String value : fieldsValue) {
					if (it.hasNext()) {
                        qfn++;
						fName=it.next();
					}
					else if (qfn!=1) {
						i++;
						fName="f"+i;
					}
					_writer.startElement(fName);
					_writer.addText(value);
					_writer.endElement();
				}
				_writer.endElement();
			}
			else {
				StringBuffer text=new StringBuffer();
				Iterator<String> it=_fieldsName.iterator();
				int i=0;
				for (String value : fieldsValue) {
					if (it.hasNext()) {
						_writer.addAttribute(it.next(),value);
					}
					else {
						text.append(value);
					}
				}
				_writer.startElement(_rowName);
				_writer.addText(text.toString());
				_writer.endElement();
			}
		}
        catch (SAXException ex) {
            _exceptions.add(ex);
        }
		
	}
}
